<?php
/**
 * Created by PhpStorm.
 * User: matheussilva
 * Date: 10/09/14
 * Time: 17:13
 */

namespace Folha\Logic\ImportacaoLoteEvento;


use Folha\Entities\FpaCalculofolha;
use Folha\Entities\FpaEventoh;
use \Folha\Entities\FpaItemlotelanc;
use \Folha\Entities\FpaLotelancevento;
use \Folha\Entities\GpeVinculom;
use \Folha\Entities\FpaItecalcfolha;
use \Folha\Repositories\FpaItemlotelanc as Itemlotelanc;
use \Folha\Repositories\FpaLotelancevento as Lotelancevento;
use \Geral\Util\Ambiente;
use \HCM\Util\DateUtil;
use \Geral\Util\NovoCodigo;
use \Setforms2\Shared\Registry;
use \Setforms2\Shared\Connection;

class LoteEvento {

    /**
     * ------------------------------------ CONSTANTES ------------------------------------
     * @const int XLS
     * @const int XLSX
     */
    const XLS   = 1;
    const XLSX  = 2;

    /**
     * ------------------------------------ ATRIBUTOS ------------------------------------
     * @var \PHPExcel $objconteudoarquivo
     * @var int $nrcnpjestrutlegal
     * @var int $nrcnpjestrutgerencial
     * @var string $nmestrutlegal
     * @var string $nmestrutgerencial
     * @var string $dtmescompetencia
     * @var array $infoarquivo
     */
    private $objconteudoarquivo;
    private $nrcnpjestrutlegal;
    private $nrcnpjestrutgerencial;
    private $nmestrutlegal;
    private $nmestrutgerencial;
    private $dtmescompetencia;
    private $infoarquivo;
    private $origem;
    private $nrlote;


    // ------------------------------------ CONSTRUTOR ------------------------------------
    public function __construct($enderecoArquivo, $nrorg, $itensLote = null, $origem = null)
    {
        $this->origem = $origem;

        try {
            /*Importações originadas das planílhas*/
            if(is_null($this->origem)){
                $extensao = $this->verificaArquivo($enderecoArquivo);
                $this->objconteudoarquivo = $this->lerArquivo($enderecoArquivo, $extensao);
                $this->infoarquivo = $this->separaInfo();
//                $validador = new ValidadorArquivo($this->infoarquivo, $nrorg);
//                $validador->validar($nrorg);
                $lote = $this->insereLoteLancEvento($enderecoArquivo);
                $this->insereIteLoteEvento($this->infoarquivo, $lote);
                unlink($enderecoArquivo);
            }
            /*Importações originadas da apuração de frequência*/
            elseif($this->origem == 'PTO'){
                $this->dtmescompetencia = \FPA::getDtCompetencia();
                $this->infoarquivo = $itensLote;
                $lote = $this->insereLoteLancEvento($enderecoArquivo);
                $this->insereIteLoteEvento($this->infoarquivo, $lote);
            }
        } catch (\Exception $e) {
            throw $e;
        }

    }

    // ------------------------------------ MÉTODOS ------------------------------------
    /**
     * VERIFICAR FORMATO DO ARQUIVO
     * @param string $arquivo
     * @return int
     * @throws $e
     */
    private function verificaArquivo($arquivo)
    {
        try {
            $nome = preg_split("/(\\|\/|\.)/", $arquivo);
            $extencao = strtoupper($nome[count($nome) - 1]);
            switch ($extencao) {
                case('XLS'):
                {
                    return self::XLS;
                }
                case('XLSX'):
                {
                    return self::XLSX;
                }
                default:
                {
                    throw new \Exception('Extensão de arquivo inválida.');
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * LER XLS
     * @param string $arquivo
     * @param string $extensao
     * @return \PHPExcel
     */
    private function lerArquivo($arquivo, $extensao)
    {
        if($extensao === self::XLS){
            $leitor = new \PHPExcel_Reader_Excel5();
        }else{
            $leitor = new \PHPExcel_Reader_Excel2007();
        }

        $leitor->setReadDataOnly(true);
        return $leitor->load($arquivo);
    }

    /**
     * SEPARAR INFORMAÇÕES
     */

    private function separaInfo()
    {
        try {
            $this->objconteudoarquivo->setActiveSheetIndex(0);
            $this->nrcnpjestrutlegal = $this->objconteudoarquivo->getActiveSheet()->getCell('B1')->getValue();
            $this->nrcnpjestrutgerencial = $this->objconteudoarquivo->getActiveSheet()->getCell('B2')->getValue();

            $this->nmestrutlegal = $this->objconteudoarquivo->getActiveSheet()->getCell('C1')->getValue();
            $this->nmestrutgerencial = $this->objconteudoarquivo->getActiveSheet()->getCell('C2')->getValue();

            $dataVerifica = $this->objconteudoarquivo->getActiveSheet()->getCell('B3')->getValue();
            $dataVerifica = DateUtil::getDataDeString(jdtojulian($dataVerifica + 2415032), DateUtil::FORMATO_AMERICANO_2, true);

            if ($dataVerifica->format('d/m/Y') != substr(\Fpa::getDtCompetencia(), 0, 10)) {
                throw new \Exception("A data de competência do arquivo é diferente da data de competência do sistema");
            }

            $this->dtmescompetencia = $dataVerifica->format('d/m/Y');

            $nrposilinha = 5;
            $nrposicoluna = 'A';

            $valorcelula = $this->objconteudoarquivo->getActiveSheet()->getCell($nrposicoluna . $nrposilinha)->getValue();
            while (!is_null($valorcelula)) {

                $nmcolunas[] = preg_split('/\s(.)\s/', utf8_decode($valorcelula));
                $nrposicoluna++;

                $valorcelula = $this->objconteudoarquivo->getActiveSheet()->getCell($nrposicoluna . $nrposilinha)->getValue();

            }

            $nrposilinha++;
            $info = array();

            $valorcelula = $this->objconteudoarquivo->getActiveSheet()->getCell('A' . $nrposilinha)->getValue();
            $valorcelula_p = $this->objconteudoarquivo->getActiveSheet()->getCell('C' . $nrposilinha)->getValue();

            while (!is_null($valorcelula) && !is_null($valorcelula_p)) {
                $infolinha = array();
                for ($c = 'A', $d = 0; $c < $nrposicoluna; $c++, $d++, $valorcelula = $this->objconteudoarquivo->getActiveSheet()->getCell($c . $nrposilinha)->getValue()) {
                    if ($c === 'A') {
                        $nrvinculom = $valorcelula;
                    } elseif ($c === 'C') {
                        $vinculo = $valorcelula;
                    } elseif ($c > 'E') {
                        if (is_array($nmcolunas[$d])) {
                            $infolinha['NRVINCULOM'] = $nrvinculom;
                            $infolinha['NMVINCULOM'] = $vinculo;
                            $infolinha['NREVENTOM'] = $nmcolunas[$d][0];
                            $infolinha['NMEVENTOH'] = $nmcolunas[$d][1];
                            $infolinha['VRREFEVENTO'] = $valorcelula;
                            $info[] = $infolinha;
                        } else {
                            $infolinha['NRVINCULOM'] = $nrvinculom;
                            $infolinha['NMVINCULOM'] = $vinculo;
                            $infolinha['NREVENTOM'] = $nmcolunas[$d];
                            $infolinha['NMEVENTOH'] = null;
                            $infolinha['VRREFEVENTO'] = $valorcelula;
                            $info[] = $infolinha;
                        }
                    }
                }

                $nrposilinha++;

                $valorcelula = $this->objconteudoarquivo->getActiveSheet()->getCell('A' . $nrposilinha)->getValue();
                $valorcelula_p = $this->objconteudoarquivo->getActiveSheet()->getCell('C'.$nrposilinha)->getValue();
            }
            return $info;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function insereLoteLancEvento($enderecoArquivo)
    {
        /*Importações originadas das planílhas*/
        if(is_null($this->origem)){
            $ult_barr = strrpos($enderecoArquivo, "\\");
            $pont = strrpos($enderecoArquivo, ".");
            $nome = substr($enderecoArquivo, $ult_barr+1, $pont) ;
            $nome = strtoupper($nome);
            $nome = "IMPORTAÇÃO DE ARQUIVO $nome";
            $origem = 'PLANILHA';
        }
        /*Importações originadas da apuração de frequência*/
        elseif($this->origem == 'PTO'){
            $nome = "IMPORTAÇAO DO RESUMO DE SITUAÇÃO DE FREQUÊNCIA";
            $origem = 'RESUSITUFREQ';
        }

//        tk_abort(DateUtil::getDataDeString(jdtojulian($this->dtmescompetencia+2415032), DateUtil::FORMATO_AMERICANO_2, true));

        try {
            $nrlotelacevento = Novocodigo::geraCodigo(Ambiente::getNrorgTrabalho(), 'FPA_LOTELANCEVENTO', 1, 22, true, false);
            $this->nrlote = $nrlotelacevento;
            $obj = new FpaLotelancevento();

            $obj->setNrlotelancevento($nrlotelacevento);
            $obj->setDslotelancevento($nome);
            $obj->setDtlotelancevento(DateUtil::getDataAtual());
            $obj->setDtcompetloteeve(DateUtil::getDataDeString($this->dtmescompetencia));
            $obj->setIdstatuslotelanc('PENDENTE');
            $obj->setDsobslotelanc('');
            $obj->setIdativo('S');
            $obj->setIdoriglotelanc($origem);
            $obj->setNrtipomoviment(1);
            $obj->setNrorginclusao(Ambiente::getNrorgTrabalho());
            $obj->setDtinclusao(DateUtil::getDataAtual(true));
            $obj->setCdoperinclusao(Ambiente::getCdOperador());
            $obj->setNrorgultatu(Ambiente::getNrorgTrabalho());
            $obj->setDtultatu(DateUtil::getDataAtual(true));
            $obj->setCdoperultatu(Ambiente::getCdOperador());
            $obj->setNrorg(Ambiente::getNrorgTrabalho());
            $obj->persist();

            //Lotelancevento::insereItem($obj);
            Registry::getEntityManager()->flush();
            return $nrlotelacevento;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function insereIteLoteEvento($infos, $lote)
    {
        try {
            foreach ($infos as $info) {
                $obj = new FpaItemlotelanc();

                //$obj->setNritlotelanc(Novocodigo::geraCodigo(Ambiente::getNrorgTrabalho(), 'FPA_ITEMLOTELANC', 1, 22, true));
                $obj->setNrlotelancevento($lote);
                $obj->setNrvinculom($info['NRVINCULOM']);
                $obj->setNreventom($info['NREVENTOM']);
                $obj->setIdativo('S');
                $obj->setNrorginclusao(Ambiente::getNrorgTrabalho());
                $obj->setDtinclusao(DateUtil::getDataAtual(false));
                $obj->setCdoperinclusao(Ambiente::getCdOperador());
                $obj->setNrorgultatu(Ambiente::getNrorgTrabalho());
                $obj->setDtultatu(DateUtil::getDataAtual(false));
                $obj->setCdoperultatu(Ambiente::getCdOperador());
                $obj->setNrorg(Ambiente::getNrorgTrabalho());

                /**
                 * @var FpaEventoh $ref
                 */
                $params = array('P_NRORG' => Ambiente::getNrorgTrabalho(),
                                'P_DTMESCOMPETENCIA' => Ambiente::getDtmescompetencia(),
                                'P_NREVENTOM' => $info['NREVENTOM']);
                $where = "(NRORG = :P_NRORG OR NRORG = 0) AND NREVENTOH IN (SELECT NREVENTOH FROM FPA_EVENTOH WHERE DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM FPA_EVENTOH MAX_EVENTOH WHERE MAX_EVENTOH.NRORG = FPA_EVENTOH.NRORG AND MAX_EVENTOH.NREVENTOM = FPA_EVENTOH.NREVENTOM AND MAX_EVENTOH.NREVENTOM = :P_NREVENTOM AND TRUNC(MAX_EVENTOH.DTMESCOMPETENC, 'DD') <= :P_DTMESCOMPETENCIA AND (MAX_EVENTOH.NRORG = :P_NRORG OR MAX_EVENTOH.NRORG = 0)))";
                $ref = FpaEventoh::getObjectByWhere($where, $params);

                if ($ref->getIdmediavalref() == 'REFERENCIA') {

                    $obj->setVrreffolhalote($info['VRREFEVENTO']);
                    $obj->setVreventolote('');
                } else {
                    $obj->setVreventolote($info['VRREFEVENTO']);
                    $obj->setVrreffolhalote('');
                }

                if (($obj->getVreventolote()) || ($obj->getVrreffolhalote())) {
                    $obj->setNritlotelanc(Novocodigo::geraCodigo(Ambiente::getNrorgTrabalho(), 'FPA_ITEMLOTELANC', 1, 22, true, false));
                    $obj->persist();
                    //Itemlotelanc::insereItem($obj);
                }

            }

            Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e;
        }
        return true;
    }

    public static function exportFolha($loteevento, $dtcompetencia, $nrorg, $nrtpmovimento, $cdoperador)
    {
        try {
            
            $nrcalculofolha = \FPA::determinaNrCalculoFolha($nrorg,$nrtpmovimento, $cdoperador);

            $params = array('P_NRORG' => $nrorg,
                            'P_NRLOTELANCEVENTO' => $loteevento,
                            'P_DTCOMPETLOTEEVE' => $dtcompetencia);
            $where = 'NRORG = :P_NRORG AND NRLOTELANCEVENTO IN (SELECT NRLOTELANCEVENTO FROM FPA_LOTELANCEVENTO WHERE NRORG = :P_NRORG AND NRLOTELANCEVENTO = :P_NRLOTELANCEVENTO AND FPA_LOTELANCEVENTO.DTCOMPETLOTEEVE = :P_DTCOMPETLOTEEVE)';
            $itens = FpaItemlotelanc::getObjectsByWhere($where, $params);

            /**
             * @var FpaItemlotelanc $item
             */
            foreach($itens as $item){

                $params = array('P_NRORG' => $nrorg,
                                'P_NRVINCULOM' => $item->getNrvinculom(),
                                'P_NRCALCULOFOLHA' => $nrcalculofolha);

                $where = "NRORG = :P_NRORG AND NRVINCULOM = :P_NRVINCULOM AND NREVENTOM = 3030 AND NRCALCULOFOLHA = :P_NRCALCULOFOLHA" ;

                $itenscalcfolha = FpaItecalcfolha::getObjectsByWhere($where, $params);

                $params = array('P_NRORG' => $nrorg,
                                'P_NRORGPADRAO' => 0,
                                'P_DTMESCOMPETENCIA' => $dtcompetencia,
                                'P_NREVENTOM' => $item->getNreventom());
                $where = "(NRORG = :P_NRORG OR NRORG = :P_NRORGPADRAO) AND NREVENTOH IN (SELECT NREVENTOH FROM FPA_EVENTOH WHERE NREVENTOM = :P_NREVENTOM AND (NRORG = :P_NRORG OR NRORG = :P_NRORGPADRAO) AND DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM FPA_EVENTOH MAX_EVENTOH WHERE MAX_EVENTOH.NREVENTOM = FPA_EVENTOH.NREVENTOM AND MAX_EVENTOH.NRORG = FPA_EVENTOH.NRORG AND TRUNC(MAX_EVENTOH.DTMESCOMPETENC, 'DD') <= :P_DTMESCOMPETENCIA))";
                $demonstra = FpaEventoh::getObjectByWhere($where, $params);
                $demonstra = $demonstra->getIdeventodemonst();

                $params = array('P_NRORG' => $nrorg,
                    'P_NRVINCULOM'     => $item->getNrvinculom(),
                    'P_NRCALCULOFOLHA' => $nrcalculofolha,
                    'P_NREVENTOM'      => $item->getNreventom(),
                    'P_DTOCORRENCIA'   => DateUtil::getDataDeString($dtcompetencia,DateUtil::FORMATO_BRASILEIRO, true));

                $where = "NRORG          = :P_NRORG
                      AND NRVINCULOM     = :P_NRVINCULOM
                      AND NREVENTOM      = :P_NREVENTOM
                      AND NRCALCULOFOLHA = :P_NRCALCULOFOLHA
                      AND DTOCORRENCIA   = :P_DTOCORRENCIA";

                $uniqueitem = FpaItecalcfolha::getObjectByWhere($where, $params);

                try {
                    if (!$itenscalcfolha) {
                        if (!$uniqueitem) {
                            $itecalcfolha = new FpaItecalcfolha();
                            $itecalcfolha->setNritemcalcfolha(Novocodigo::geraCodigo(Ambiente::getNrorgTrabalho(), 'FPA_ITECALCFOLHA', 1, 22, true));
                            $itecalcfolha->setNrcalculofolha($nrcalculofolha);
                            $itecalcfolha->setNrvinculom($item->getNrvinculom());
                            $itecalcfolha->setNreventom($item->getNreventom());
                            $itecalcfolha->setNrorg(Ambiente::getNrorgTrabalho());
                            $itecalcfolha->setIdevendemoncalc($demonstra);
                            $itecalcfolha->setVrreffolha($item->getVrreffolhalote());
                            $itecalcfolha->setVreventofolha($item->getVreventolote());
                            $itecalcfolha->setQtocorproxperi(1);
                            $itecalcfolha->setIdrefevalorcalc('NORMAL');
                            $itecalcfolha->setDtinclusao(DateUtil::getDataAtual(false));
                            $itecalcfolha->setDtultatu(DateUtil::getDataAtual(false));
                            $itecalcfolha->setNrorginclusao(Ambiente::getNrorgTrabalho());
                            $itecalcfolha->setCdoperinclusao(Ambiente::getCdOperador());
                            $itecalcfolha->setNrorgultatu(Ambiente::getNrorgTrabalho());
                            $itecalcfolha->setCdoperultatu(Ambiente::getCdOperador());
                            $itecalcfolha->setDtocorrencia(DateUtil::getDataDeString($dtcompetencia, DateUtil::FORMATO_BRASILEIRO, true));
                            $itecalcfolha->setNrlotelancevento($loteevento);

                            $itecalcfolha->persist();
                        } else {
                            throw new \Exception('Evento '.$item->getNreventom(). ' já existe p/ o vínculo '.$item->getNrvinculom().'.', '20');
                        }
                    }else{
                        throw new \Exception('Já existe líquido para o vínculo '.$item->getNrvinculom(), '10');
                    }
                } catch (\Exception $e) {
                    if($e->getCode() == '10'){
                        tk_message($e->getMessage());
                    }else{
                        if($e->getCode() == '20'){
                            tk_message($e->getMessage());
                        } else {
                            throw $e;
                        }
                    }
                }
            }
            \Setforms2\Shared\Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            if($e->getCode() == '10'){
                tk_message($e->getMessage());
            }else{
                throw $e;
            }
        }
    }

    public function getNrlote()
    {
        return $this->nrlote;
    }

    public static function cancelaLote($nrlotelancevento, $dtmescompetencia, $nrtipomoviment, $nrorg)
    {
        try {

            $params = array('P_NRORG' => $nrorg,
                            'P_DTOCORRENCIA' => $dtmescompetencia,
                            'P_NRTIPOMOVIMENT' => $nrtipomoviment,
                            'P_NRLOTELANCEVENTO' => $nrlotelancevento);
            $where = "NRORG = :P_NRORG AND DTOCORRENCIA = :P_DTOCORRENCIA AND NRTIPOMOVIMENT = :P_NRTIPOMOVIMENT AND NRCALCULOFOLHA IN (SELECT NRCALCULOFOLHA FROM FPA_ITECALCFOLHA WHERE NRORG = :P_NRORG AND NRLOTELANCEVENTO = :P_NRLOTELANCEVENTO)" ;
            $fpacalculofolha = FpaCalculofolha::getObjectsByWhere($where, $params);

            /**
             * @var FpaCalculofolha $calculofolha
             */
            $calculofolha = $fpacalculofolha[0];

            $params = array('P_NRORG' => $nrorg,
                            'P_NRLOTELANCEVENTO' => $nrlotelancevento,
                            'P_DTCOMPETLOTEEVE' => $dtmescompetencia);
            $where = 'NRORG = :P_NRORG AND NRLOTELANCEVENTO IN (SELECT NRLOTELANCEVENTO FROM FPA_LOTELANCEVENTO WHERE NRORG = :P_NRORG AND NRLOTELANCEVENTO = :P_NRLOTELANCEVENTO AND FPA_LOTELANCEVENTO.DTCOMPETLOTEEVE = :P_DTCOMPETLOTEEVE)';
            $itenslanclote = FpaItemlotelanc::getObjectsByWhere($where, $params);

            /**
             * @var FpaItemlotelanc $itemlanclote
             */
            foreach ($itenslanclote as $itemlanclote) {

                /*Verifica se existe liquido para o vinculo*/
                $params = array('P_NRORG' => $nrorg,
                                'P_NRVINCULOM' => $itemlanclote->getNrvinculom(),
                                'P_NRCALCULOFOLHA' => $calculofolha->getNrcalculofolha());
                $where = "NRORG = :P_NRORG AND NRCALCULOFOLHA = :P_NRCALCULOFOLHA AND NRVINCULOM = :P_NRVINCULOM AND NREVENTOM IN (3030,3050)";
                $itensliquido = FpaItecalcfolha::getObjectsByWhere($where, $params);

                try {
                    if (!$itensliquido) {

                        /*Se não houver líquido busca os itens de cálculo referentes ao vínculo e ao lote e apaga um a um*/
                        $params = array('P_NRORG' => $nrorg,
                                        'P_NRCALCULOFOLHA' => $calculofolha->getNrcalculofolha(),
                                        'P_NRVINCULOM' => $itemlanclote->getNrvinculom(),
                                        'P_NRLOTELANCEVENTO' => $nrlotelancevento);
                        $where = "NRORG = :P_NRORG AND NRCALCULOFOLHA = :P_NRCALCULOFOLHA AND NRLOTELANCEVENTO = :P_NRLOTELANCEVENTO";
                        $itenscalculo = FpaItecalcfolha::getObjectsByWhere($where, $params);

                        /**
                         * @var FpaItecalcfolha $itemcalc
                         */
                        foreach ($itenscalculo as $itemcalc) {
                            $itemcalc->delete();
                        }
                    } else {
                        throw new \Exception('Já existe líquido para o vinculo '.$itemlanclote->getNrvinculom(), '10');
                    }
                } catch (\Exception $e) {
                    if($e->getCode() == '10'){
                        tk_message($e->getMessage());
                    }else{
                        throw $e;
                    }
                }
            }

            /*Atualiza o status do lote de eventos para pendente*/
            /**
             * @var FpaLotelancevento $obj
             */
            $obj = FpaLotelancevento::getObjectByWhere('NRORG = ? AND NRLOTELANCEVENTO = ?', array($nrorg, $nrlotelancevento));
            $obj->setIdstatuslotelanc('PENDENTE');
            $obj->persist();

            \Setforms2\Shared\Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            if($e->getCode() == '10'){
                tk_message($e->getMessage());
            }else{
                throw $e;
            }
        }
    }
} 